class Librarix::Models::Movie < Sequel::Model
  def self.popular
    unrestrict_primary_key

    Tmdb::Search.new('/movie/popular').fetch.map do |m|
      find_or_create(id: m['id']) { |movie| movie.data = m }
    end
  end

  def self.find_or_fetch(id)
    find(id: id) || fetch(id)
  end

  def self.fetch(id)
    unrestrict_primary_key

    movie = find(id: id) || new(id: id)
    movie.fetch!
  end

  plugin :serialization
  serialize_attributes :json, :data

  many_to_many :genres
  plugin :association_dependencies, genres: :nullify

  def fetch!
    tmdb_data = Tmdb::Movie.detail(id)
    return if tmdb_data['status_code'] == 34

    self.data = tmdb_data
    save

    genres_id = data['genres'].map { |g| g['id'] }

    Librarix::Models::Genre.fetch if Librarix::Models::Genre.empty?

    Librarix::Models::Genre.where(id: genres_id).all.each do |genre|
      add_genre(genre) unless genres.include?(genre)
    end

    self
  end

  def add!
    self.added = true
    save
  end

  def download!
    self.downloaded = !downloaded
    save
  end

  def view!
    self.viewed = !viewed
    save
  end
end
